#include "dictformatter.h"
#include "dict.h"

#include <QRegExp>

void tags(QString &text)
{
	text.replace('<', "&lt;");
	text.replace('>', "&gt;");
}

QString links(const QString &text)
{
	QString html = text;
	// replace html tags
	tags(html);

	int start = 0;
	while ((start = html.indexOf('{', start + 1)) >= 0) {
		int end = html.indexOf('}', start);
		// missing matching brace
		if (end < 0) break;

		int len = end - start;
		// too long for a link
		if (len > 240) continue;

		QString link = html.mid(start + 1, len - 1);
		// multiple lines inside braces, could be source code
		if (link.count('\n') > 1) continue;

		int pos;
		// skip external links
		if ((pos = link.indexOf('(')) >= 0 && link.indexOf(')') > pos
			&& len - pos > 5) {
				html.replace(start, len + 1, link);
				continue;
		}
		// format link
		html.replace(start, len + 1, QString("<a href=\"%1\">%2</a>")
			.arg(link.left(pos).simplified(), link));
	}
	return html;
}

QString freedict(const QString &text)
{
	QString html = text;

	QRegExp slashes("/([^\n\\.]+)/");
	html.replace(slashes, "[\\1]");

	QRegExp defn("^([^\n\\[]+)");
	html.replace(defn, "<dt><span class=\"hword\">\\1</span>");

	QRegExp pron("(\\[[^\n\\.]+\\])");
	html.replace(pron, "<span class=\"pronc\">\\1</span> ");

	QRegExp desc("\n\\s+(\\S+)");
	html.replace(desc, "<dd>\\1");

	return html;
}

QString beslov(const QString &text)
{
	QString html = text;

	QRegExp defn("^\\s+");
	html.replace(defn, "<dt>");

	return html;
}

QString ozhegov(const QString &text)
{
	QString html = text;
	tags(html);

	QRegExp defn("^([^\n]+)");
	html.replace(defn, "<dt class=\"hword\">\\1<dd>");

	QRegExp exmpl("\n\\s+(E.g.)");
	html.replace(exmpl, "<dd>\\1");

	return html;
}

QString larousse(const QString &text)
{
	QString html = text;
	tags(html);

	QRegExp defn("^([^\n]+)");
	html.replace(defn, "<dt class=\"hword\">\\1<dt class=\"chap\">");

	QRegExp chap("\n\\s+([IVX]+\\.)");
	html.replace(chap, "<dt class=\"chap\">\\1");

	QRegExp sect("\n\\s+(\\d+\\.)");
	html.replace(sect, "<dt class=\"sect\">\\1");

	return html;
}

QString mueller(const QString &text)
{
	QString html = text;

	QRegExp abrv("_(\\S+\\.)");
	html.replace(abrv, "<span class=\"abbrv\">\\1</span>");

	QRegExp chap("_(\\S+\\s)");
	html.replace(chap, "<dt class=\"chap\">\\1");

	chap.setPattern("\n\\s+([IVX]+\\s\\[)");
	html.replace(chap, "<dt class=\"chap\">\\1");

	QRegExp pron("(\\[[^\n<]+\\])");
	html.replace(pron, "<span class=\"pronc\">\\1</span> ");

	QRegExp defn("^([^\n]+)");
	html.replace(defn, "<dt class=\"hword\">\\1<dt class=\"sect\">");

	QRegExp sect("\n\\s+(\\d+\\.)");
	html.replace(sect, "<dt class=\"sect\">\\1");

	QRegExp desc("\n\\s+(\\d+[:\\)]\\s+)");
	html.replace(desc, "<dd>\\1");

	return html;
}

QString foldoc(const QString &text)
{
	QString html = links(text);

	QRegExp defn("^([^\n]+)");
	html.replace(defn, "<dt class=\"hword\">\\1");

	QRegExp desc("\n\\s*\n");
	html.replace(desc, "<dd class=\"para\">\n");

	return html;
}

QString wordnet(const QString &text)
{
	QString html = links(text);

	QRegExp defn("^([^\n]+)");
	html.replace(defn, "<dt class=\"hword\">\\1<dd>");

	QRegExp desc("\n\\s+(\\d+:\\s+)");
	html.replace(desc, "<dd class=\"defn\">\\1");

	QRegExp abrv("\n\\s+(\\w+)(\\s\\d*:\\s+)");
	html.replace(abrv, "<dd class=\"defn\"><span class=\"abbrv\">\\1</span>\\2");

	QRegExp also("\n\\s+(\\[also:)");
	html.replace(also, "<dd>\\1");

	return html;
}

QString xdxf(const QString &text)
{
	QString html = text;

	QRegExp defn("^<k>(.+)</k>");
	html.replace(defn, "<dt class=\"hword\">\\1<dd>");

	QRegExp abrv("<abr>(.+)</abr>");
	html.replace(abrv, "<span class=\"abbrv\">\\1</span>");

	QRegExp pron("<tr>(.+)</tr>");
	html.replace(pron, "<span class=\"pronc\">[\\1]</span>");

	return html;
}

bool isxdxf(const QString &text)
{
	return text.contains("<k>") && text.contains("</k>");
}

DictFormatter::DictFormatter()
{
	fmap["freedict"] = freedict;
	fmap["mueller"] = mueller;
	fmap["beslov"] = beslov;
	fmap["ozhegov"] = ozhegov;
	fmap["larousse"] = larousse;
	fmap["foldoc"] = foldoc;
	fmap["wordnet"] = wordnet;
	fmap["xdxf"] = xdxf;

	maxLen = 12000;
}

QString DictFormatter::format(const DictResponse &response)
{
	QString html;
	formatfunc func = 0;

	if (fmap.contains(response.database))
		func = fmap.value(response.database);
	else {
		foreach (QString key, fmap.keys())
			if (response.database.contains(key)
			|| response.description.contains(key, Qt::CaseInsensitive)) {
				func = fmap.value(key);
				break;
			}
		if (!func && isxdxf(response.defines.first()))
			func = xdxf;
		fmap[response.database] = func;
	}
	if (!func)
		return words(response.defines);

	html = (QString("<div class=\"%1\">").arg(response.database));
	foreach (QString text, response.defines) {
		text.truncate(maxLen);
		html += "<dl class=\"descr\">";
		html += func(text);
		html += "</dl>";
	}
	html += "</div>";
	return html;
}

QString DictFormatter::words(const QStringList &list)
{
	QString html;
	foreach (QString text, list) {
		text.truncate(maxLen);
		html += words(text);
	}
	return html;
}

QString DictFormatter::words(const QString &text)
{
	QString html;
	html += "<div class=\"define\">";
	html += links(text);
	html += "</div>";
	return html;
}
